@@ -1,14 +1,21 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * libmm-glib -- Access modem status & information from glib applications
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details:
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  * Copyright (C) 2012 Lanedo GmbH <aleksander@lanedo.com>
  */
@@ -52,7 +59,7 @@ check_append_or_replace (MMLocationGpsNmea *self,
     GMatchInfo *match_info = NULL;
 
     if (G_UNLIKELY (!self->priv->sequence_regex))
-        self->priv->sequence_regex = g_regex_new ("\\$GPGSV,(\\d),(\\d).*",
+        self->priv->sequence_regex = g_regex_new ("\\$..(?:ALM|GSV|RTE|SFI),(\\d),(\\d).*",
                                                   G_REGEX_RAW | G_REGEX_OPTIMIZE,
                                                   0,
                                                   NULL);
@@ -176,6 +183,8 @@ mm_location_gps_nmea_get_traces (MMLocationGpsNmea *self)
 {
     GPtrArray *built = NULL;
 
+    g_return_val_if_fail (MM_IS_LOCATION_GPS_NMEA (self), NULL);
+
     g_hash_table_foreach (self->priv->traces,
                           (GHFunc)build_all_foreach,
                           &built);
@@ -188,49 +197,6 @@ mm_location_gps_nmea_get_traces (MMLocationGpsNmea *self)
 
 /*****************************************************************************/
 
-#ifndef MM_DISABLE_DEPRECATED
-
-static void
-build_full_foreach (const gchar *trace_type,
-                    const gchar *trace,
-                    GString **built)
-{
-    if ((*built)->len == 0 || g_str_has_suffix ((*built)->str, "\r\n"))
-        g_string_append (*built, trace);
-    else
-        g_string_append_printf (*built, "\r\n%s", trace);
-}
-
-/**
- * mm_location_gps_nmea_build_full:
- * @self: a #MMLocationGpsNmea.
- *
- * Gets a compilation of all cached traces, in a single string.
- * Traces are separated by '\r\n'.
- *
- * Returns: (transfer full): a string containing all traces, or #NULL if none
- * available. The returned value should be freed with g_free().
- *
- * Since: 1.0
- * Deprecated: 1.14: user should use mm_location_gps_nmea_get_traces() instead,
- * which provides a much more generic interface to the full list of traces.
- */
-gchar *
-mm_location_gps_nmea_build_full (MMLocationGpsNmea *self)
-{
-    GString *built;
-
-    built = g_string_new ("");
-    g_hash_table_foreach (self->priv->traces,
-                          (GHFunc)build_full_foreach,
-                          &built);
-    return g_string_free (built, FALSE);
-}
-
-#endif
-
-/*****************************************************************************/
-
 /**
  * mm_location_gps_nmea_get_string_variant: (skip)
  */
@@ -243,8 +209,9 @@ mm_location_gps_nmea_get_string_variant (MMLocationGpsNmea *self)
     g_return_val_if_fail (MM_IS_LOCATION_GPS_NMEA (self), NULL);
 
     traces = mm_location_gps_nmea_get_traces (self);
-    built = g_strjoinv ("\r\n", traces);
-    return g_variant_ref_sink (g_variant_new_string (built));
+    if (traces)
+        built = g_strjoinv ("\r\n", traces);
+    return g_variant_ref_sink (g_variant_new_string (built ? built : ""));
 }
 
 /*****************************************************************************/
