@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- * libmm -- Access modem status & information from glib applications
+ * libmm-glib -- Access modem status & information from glib applications
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -89,9 +89,8 @@ gboolean           mm_modem_get_supported_capabilities (MMModem *self,
 
 MMModemCapability  mm_modem_get_current_capabilities (MMModem *self);
 
-guint              mm_modem_get_max_bearers          (MMModem *self);
-
-guint              mm_modem_get_max_active_bearers   (MMModem *self);
+guint              mm_modem_get_max_active_bearers             (MMModem *self);
+guint              mm_modem_get_max_active_multiplexed_bearers (MMModem *self);
 
 const gchar * const *mm_modem_get_bearer_paths       (MMModem *self);
 gchar              **mm_modem_dup_bearer_paths       (MMModem *self);
