@@ -1,14 +1,21 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * libmm-glib -- Access modem status & information from glib applications
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details:
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  * Copyright (C) 2011 Aleksander Morgado <aleksander@gnu.org>
  */
@@ -34,7 +41,7 @@
  * mm_modem_messaging_create() or mm_modem_messaging_create_sync().
  */
 
-G_DEFINE_TYPE (MMSmsProperties, mm_sms_properties, G_TYPE_OBJECT);
+G_DEFINE_TYPE (MMSmsProperties, mm_sms_properties, G_TYPE_OBJECT)
 
 #define PROPERTY_TEXT                    "text"
 #define PROPERTY_DATA                    "data"
@@ -43,6 +50,8 @@ G_DEFINE_TYPE (MMSmsProperties, mm_sms_properties, G_TYPE_OBJECT);
 #define PROPERTY_VALIDITY                "validity"
 #define PROPERTY_CLASS                   "class"
 #define PROPERTY_DELIVERY_REPORT_REQUEST "delivery-report-request"
+#define PROPERTY_TELESERVICE_ID          "teleservice-id"
+#define PROPERTY_SERVICE_CATEGORY        "service-category"
 
 struct _MMSmsPropertiesPrivate {
     gchar *text;
@@ -54,6 +63,8 @@ struct _MMSmsPropertiesPrivate {
     gint class;
     gboolean delivery_report_request_set;
     gboolean delivery_report_request;
+    MMSmsCdmaTeleserviceId teleservice_id;
+    MMSmsCdmaServiceCategory service_category;
 };
 
 /*****************************************************************************/
@@ -64,6 +75,8 @@ struct _MMSmsPropertiesPrivate {
  * @text: The text to set, in UTF-8.
  *
  * Sets the message text.
+ *
+ * Since: 1.0
  */
 void
 mm_sms_properties_set_text (MMSmsProperties *self,
@@ -81,7 +94,11 @@ mm_sms_properties_set_text (MMSmsProperties *self,
  *
  * Gets the message text, in UTF-8.
  *
- * Returns: (transfer none): The message text, or %NULL if it doesn't contain any (e.g. contains data instead). Do not free the returned value, it is owned by @self.
+ * Returns: (transfer none): The message text, or %NULL if it doesn't contain
+ * any (e.g. contains data instead). Do not free the returned value, it is
+ * owned by @self.
+ *
+ * Since: 1.0
  */
 const gchar *
 mm_sms_properties_get_text (MMSmsProperties *self)
@@ -100,6 +117,8 @@ mm_sms_properties_get_text (MMSmsProperties *self)
  * @data_length: Length of @data.
  *
  * Sets the message data.
+ *
+ * Since: 1.0
  */
 void
 mm_sms_properties_set_data (MMSmsProperties *self,
@@ -122,9 +141,12 @@ mm_sms_properties_set_data (MMSmsProperties *self,
 /**
  * mm_sms_properties_set_data_bytearray:
  * @self: A #MMSmsProperties.
- * @data: A #GByteArray with the data to set. This method takes a new reference of @data.
+ * @data: A #GByteArray with the data to set. This method takes a new reference
+ *  of @data.
  *
  * Sets the message data.
+ *
+ * Since: 1.0
  */
 void
 mm_sms_properties_set_data_bytearray (MMSmsProperties *self,
@@ -145,7 +167,10 @@ mm_sms_properties_set_data_bytearray (MMSmsProperties *self,
  *
  * Gets the message data.
  *
- * Returns: (transfer none): The message data, or %NULL if it doesn't contain any (e.g. contains text instead).
+ * Returns: (transfer none): The message data, or %NULL if it doesn't contain
+ * any (e.g. contains text instead).
+ *
+ * Since: 1.0
  */
 const guint8 *
 mm_sms_properties_get_data (MMSmsProperties *self,
@@ -153,10 +178,10 @@ mm_sms_properties_get_data (MMSmsProperties *self,
 {
     g_return_val_if_fail (MM_IS_SMS_PROPERTIES (self), NULL);
 
-    if (self->priv->data && data_len)
-        *data_len = self->priv->data->len;
+    if (data_len)
+        *data_len = (self->priv->data ? self->priv->data->len : 0);
 
-    return self->priv->data->data;
+    return (self->priv->data ? self->priv->data->data : NULL);
 }
 
 /**
@@ -165,7 +190,11 @@ mm_sms_properties_get_data (MMSmsProperties *self,
  *
  * Gets the message data.
  *
- * Returns: (transfer none): A #GByteArray with the message data, or %NULL if it doesn't contain any (e.g. contains text instead). Do not free the returned value, it is owned by @self.
+ * Returns: (transfer none): A #GByteArray with the message data, or %NULL if it
+ * doesn't contain any (e.g. contains text instead). Do not free the returned
+ * value, it is owned by @self.
+ *
+ * Since: 1.0
  */
 GByteArray *
 mm_sms_properties_peek_data_bytearray (MMSmsProperties *self)
@@ -181,7 +210,11 @@ mm_sms_properties_peek_data_bytearray (MMSmsProperties *self)
  *
  * Gets the message data.
  *
- * Returns: (transfer full): A #GByteArray with the message data, or %NULL if it doesn't contain any (e.g. contains text instead). The returned value should be freed with g_byte_array_unref().
+ * Returns: (transfer full): A #GByteArray with the message data, or %NULL if it
+ * doesn't contain any (e.g. contains text instead). The returned value should
+ * be freed with g_byte_array_unref().
+ *
+ * Since: 1.0
  */
 GByteArray *
 mm_sms_properties_get_data_bytearray (MMSmsProperties *self)
@@ -199,6 +232,8 @@ mm_sms_properties_get_data_bytearray (MMSmsProperties *self)
  * @number: The number.
  *
  * Sets the number to which the message is addressed.
+ *
+ * Since: 1.0
  */
 void
 mm_sms_properties_set_number (MMSmsProperties *self,
@@ -216,7 +251,10 @@ mm_sms_properties_set_number (MMSmsProperties *self,
  *
  * Gets the number to which the message is addressed.
  *
- * Returns: (transfer none): The number, or %NULL if it couldn't be retrieved. Do not free the returned value, it is owned by @self.
+ * Returns: (transfer none): The number, or %NULL if it couldn't be retrieved.
+ * Do not free the returned value, it is owned by @self.
+ *
+ * Since: 1.0
  */
 const gchar *
 mm_sms_properties_get_number (MMSmsProperties *self)
@@ -234,6 +272,8 @@ mm_sms_properties_get_number (MMSmsProperties *self)
  * @smsc: The SMSC number.
  *
  * Sets the SMS service center number.
+ *
+ * Since: 1.0
  */
 void
 mm_sms_properties_set_smsc (MMSmsProperties *self,
@@ -251,7 +291,10 @@ mm_sms_properties_set_smsc (MMSmsProperties *self,
  *
  * Gets the SMS service center number.
  *
- * Returns: (transfer none): The number of the SMSC, or %NULL if it couldn't be retrieved. Do not free the returned value, it is owned by @self.
+ * Returns: (transfer none): The number of the SMSC, or %NULL if it couldn't be
+ * retrieved. Do not free the returned value, it is owned by @self.
+ *
+ * Since: 1.0
  */
 const gchar *
 mm_sms_properties_get_smsc (MMSmsProperties *self)
@@ -269,6 +312,8 @@ mm_sms_properties_get_smsc (MMSmsProperties *self)
  * @validity: The validity of %MM_SMS_VALIDITY_TYPE_RELATIVE type.
  *
  * Sets the relative validity time of the SMS.
+ *
+ * Since: 1.0
  */
 void
 mm_sms_properties_set_validity_relative (MMSmsProperties *self,
@@ -287,6 +332,8 @@ mm_sms_properties_set_validity_relative (MMSmsProperties *self,
  * Gets the relative validity type the SMS.
  *
  * Returns: a #MMSmsValidityType.
+ *
+ * Since: 1.0
  */
 MMSmsValidityType
 mm_sms_properties_get_validity_type (MMSmsProperties *self)
@@ -303,6 +350,8 @@ mm_sms_properties_get_validity_type (MMSmsProperties *self)
  * Gets the relative validity time of the SMS.
  *
  * Returns: the validity time or 0 if unknown.
+ *
+ * Since: 1.0
  */
 guint
 mm_sms_properties_get_validity_relative (MMSmsProperties *self)
@@ -318,18 +367,20 @@ mm_sms_properties_get_validity_relative (MMSmsProperties *self)
 /**
  * mm_sms_properties_set_class:
  * @self: A #MMSmsProperties.
- * @class: The message class (0..3), or -1 for invalid/unset class.
+ * @message_class: The message class (0..3), or -1 for invalid/unset class.
  *
  * Sets the 3GPP message class of the SMS.
+ *
+ * Since: 1.0
  */
 void
 mm_sms_properties_set_class (MMSmsProperties *self,
-                             gint class)
+                             gint message_class)
 {
     g_return_if_fail (MM_IS_SMS_PROPERTIES (self));
-    g_return_if_fail (class >= -1 && class <= 3);
+    g_return_if_fail (message_class >= -1 && message_class <= 3);
 
-    self->priv->class = class;
+    self->priv->class = message_class;
 }
 
 /**
@@ -339,6 +390,8 @@ mm_sms_properties_set_class (MMSmsProperties *self,
  * Gets the 3GPP message class of the SMS.
  *
  * Returns: the message class, or -1 for invalid/unset class.
+ *
+ * Since: 1.0
  */
 gint
 mm_sms_properties_get_class (MMSmsProperties *self)
@@ -356,6 +409,8 @@ mm_sms_properties_get_class (MMSmsProperties *self)
  * @request: %TRUE if delivery report is requested, %FALSE otherwise.
  *
  * Sets whether delivery report is requested for the SMS.
+ *
+ * Since: 1.0
  */
 void
 mm_sms_properties_set_delivery_report_request (MMSmsProperties *self,
@@ -374,6 +429,8 @@ mm_sms_properties_set_delivery_report_request (MMSmsProperties *self,
  * Checks whether delivery report is requested for the SMS.
  *
  * Returns: %TRUE if delivery report is requested, %FALSE otherwise.
+ *
+ * Since: 1.0
  */
 gboolean
 mm_sms_properties_get_delivery_report_request (MMSmsProperties *self)
@@ -385,6 +442,85 @@ mm_sms_properties_get_delivery_report_request (MMSmsProperties *self)
 
 /*****************************************************************************/
 
+/**
+ * mm_sms_properties_set_teleservice_id:
+ * @self: A #MMSmsProperties.
+ * @teleservice_id: The CDMA teleservice ID.
+ *
+ * Sets the CDMA teleservice ID of the SMS.
+ *
+ * Since: 1.2
+ */
+void
+mm_sms_properties_set_teleservice_id (MMSmsProperties *self,
+                                      MMSmsCdmaTeleserviceId teleservice_id)
+{
+    g_return_if_fail (MM_IS_SMS_PROPERTIES (self));
+
+    self->priv->teleservice_id = teleservice_id;
+}
+
+/**
+ * mm_sms_properties_get_teleservice_id:
+ * @self: A #MMSmsProperties.
+ *
+ * Gets the CDMA teleservice ID of the SMS.
+ *
+ * Returns: the CDMA teleservice ID.
+ *
+ * Since: 1.2
+ */
+MMSmsCdmaTeleserviceId
+mm_sms_properties_get_teleservice_id (MMSmsProperties *self)
+{
+    g_return_val_if_fail (MM_IS_SMS_PROPERTIES (self), MM_SMS_CDMA_TELESERVICE_ID_UNKNOWN);
+
+    return self->priv->teleservice_id;
+}
+
+/*****************************************************************************/
+
+/**
+ * mm_sms_properties_set_service_category:
+ * @self: A #MMSmsProperties.
+ * @service_category: The CDMA service category.
+ *
+ * Sets the CDMA service category of the SMS.
+ *
+ * Since: 1.2
+ */
+void
+mm_sms_properties_set_service_category (MMSmsProperties *self,
+                                        MMSmsCdmaServiceCategory service_category)
+{
+    g_return_if_fail (MM_IS_SMS_PROPERTIES (self));
+
+    self->priv->service_category = service_category;
+}
+
+/**
+ * mm_sms_properties_get_service_category:
+ * @self: A #MMSmsProperties.
+ *
+ * Gets the CDMA message service category of the SMS.
+ *
+ * Returns: the CDMA service category.
+ *
+ * Since: 1.2
+ */
+MMSmsCdmaServiceCategory
+mm_sms_properties_get_service_category (MMSmsProperties *self)
+{
+    g_return_val_if_fail (MM_IS_SMS_PROPERTIES (self), MM_SMS_CDMA_SERVICE_CATEGORY_UNKNOWN);
+
+    return self->priv->service_category;
+}
+
+/*****************************************************************************/
+
+/**
+ * mm_sms_properties_get_dictionary: (skip)
+ */
 GVariant *
 mm_sms_properties_get_dictionary (MMSmsProperties *self)
 {
@@ -446,6 +582,18 @@ mm_sms_properties_get_dictionary (MMSmsProperties *self)
                                PROPERTY_DELIVERY_REPORT_REQUEST,
                                g_variant_new_boolean (self->priv->delivery_report_request));
 
+    if (self->priv->teleservice_id != MM_SMS_CDMA_TELESERVICE_ID_UNKNOWN)
+        g_variant_builder_add (&builder,
+                               "{sv}",
+                               PROPERTY_TELESERVICE_ID,
+                               g_variant_new_uint32 (self->priv->teleservice_id));
+
+    if (self->priv->service_category != MM_SMS_CDMA_SERVICE_CATEGORY_UNKNOWN)
+        g_variant_builder_add (&builder,
+                               "{sv}",
+                               PROPERTY_SERVICE_CATEGORY,
+                               g_variant_new_uint32 (self->priv->service_category));
+
     return g_variant_ref_sink (g_variant_builder_end (&builder));
 }
 
@@ -539,7 +687,29 @@ consume_string (MMSmsProperties *self,
         }
 
         mm_sms_properties_set_delivery_report_request (self, request);
-    }  else if (g_str_equal (key, PROPERTY_DATA)) {
+    } else if (g_str_equal (key, PROPERTY_TELESERVICE_ID)) {
+        MMSmsCdmaTeleserviceId teleservice_id;
+        GError *inner_error = NULL;
+
+        teleservice_id = mm_common_get_sms_cdma_teleservice_id_from_string (value, &inner_error);
+        if (inner_error) {
+            g_propagate_error (error, inner_error);
+            return FALSE;
+        }
+
+        mm_sms_properties_set_teleservice_id (self, teleservice_id);
+    } else if (g_str_equal (key, PROPERTY_SERVICE_CATEGORY)) {
+        MMSmsCdmaServiceCategory service_category;
+        GError *inner_error = NULL;
+
+        service_category = mm_common_get_sms_cdma_service_category_from_string (value, &inner_error);
+        if (inner_error) {
+            g_propagate_error (error, inner_error);
+            return FALSE;
+        }
+
+        mm_sms_properties_set_service_category (self, service_category);
+    } else if (g_str_equal (key, PROPERTY_DATA)) {
         g_set_error (error,
                      MM_CORE_ERROR,
                      MM_CORE_ERROR_INVALID_ARGS,
@@ -574,6 +744,9 @@ key_value_foreach (const gchar *key,
                            &ctx->error);
 }
 
+/**
+ * mm_sms_properties_new_from_string: (skip)
+ */
 MMSmsProperties *
 mm_sms_properties_new_from_string (const gchar *str,
                                    GError **error)
@@ -648,6 +821,14 @@ consume_variant (MMSmsProperties *properties,
         mm_sms_properties_set_delivery_report_request (
             properties,
             g_variant_get_boolean (value));
+    else if (g_str_equal (key, PROPERTY_TELESERVICE_ID))
+        mm_sms_properties_set_teleservice_id (
+            properties,
+            g_variant_get_uint32 (value));
+    else if (g_str_equal (key, PROPERTY_SERVICE_CATEGORY))
+        mm_sms_properties_set_service_category (
+            properties,
+            g_variant_get_uint32 (value));
     else {
         /* Set error */
         g_set_error (error,
@@ -661,6 +842,9 @@ consume_variant (MMSmsProperties *properties,
     return TRUE;
 }
 
+/**
+ * mm_sms_properties_new_from_dictionary: (skip)
+ */
 MMSmsProperties *
 mm_sms_properties_new_from_dictionary (GVariant *dictionary,
                                        GError **error)
@@ -708,31 +892,14 @@ mm_sms_properties_new_from_dictionary (GVariant *dictionary,
 
 /*****************************************************************************/
 
-MMSmsProperties *
-mm_sms_properties_dup (MMSmsProperties *orig)
-{
-    GVariant *dict;
-    MMSmsProperties *copy;
-    GError *error = NULL;
-
-    g_return_val_if_fail (MM_IS_SMS_PROPERTIES (orig), NULL);
-
-    dict = mm_sms_properties_get_dictionary (orig);
-    copy = mm_sms_properties_new_from_dictionary (dict, &error);
-    g_assert_no_error (error);
-    g_variant_unref (dict);
-
-    return copy;
-}
-
-/*****************************************************************************/
-
 /**
  * mm_sms_properties_new:
  *
  * Creates a new empty #MMSmsProperties.
  *
  * Returns: (transfer full): a #MMSmsProperties. The returned value should be freed with g_object_unref().
+ *
+ * Since: 1.0
  */
 MMSmsProperties *
 mm_sms_properties_new (void)
@@ -748,6 +915,8 @@ mm_sms_properties_init (MMSmsProperties *self)
                                               MMSmsPropertiesPrivate);
     self->priv->validity_type = MM_SMS_VALIDITY_TYPE_UNKNOWN;
     self->priv->class = -1;
+    self->priv->teleservice_id = MM_SMS_CDMA_TELESERVICE_ID_UNKNOWN;
+    self->priv->service_category = MM_SMS_CDMA_SERVICE_CATEGORY_UNKNOWN;
 }
 
 static void
