@@ -63,7 +63,7 @@ get_usb_interface_config_index (MMPort  *data,
     guint usb_iface_num;
     guint i;
 
-    usb_iface_num = mm_kernel_device_get_property_as_int_hex (mm_port_peek_kernel_device (data), "ID_USB_INTERFACE_NUM");
+    usb_iface_num = (guint) mm_kernel_device_get_interface_number (mm_port_peek_kernel_device (data));
 
     for (i = 0; i < G_N_ELEMENTS (usb_interface_configs); i++) {
         if (usb_interface_configs[i].usb_iface_num == usb_iface_num)
@@ -132,14 +132,21 @@ out:
 
 static void
 load_connection_status_by_cid (MMBroadbandBearerCinterion *bearer,
-                               guint                       cid,
+                               gint                        cid,
                                GAsyncReadyCallback         callback,
                                gpointer                    user_data)
 {
-    GTask       *task;
-    MMBaseModem *modem;
+    GTask                  *task;
+    g_autoptr(MMBaseModem)  modem = NULL;
 
     task = g_task_new (bearer, NULL, callback, user_data);
+    if (cid == MM_3GPP_PROFILE_ID_UNKNOWN) {
+        g_task_return_new_error (task, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                                 "Unknown profile id to check connection status");
+        g_object_unref (task);
+        return;
+    }
+
     g_task_set_task_data (task, GUINT_TO_POINTER (cid), NULL);
 
     g_object_get (bearer,
@@ -152,7 +159,6 @@ load_connection_status_by_cid (MMBroadbandBearerCinterion *bearer,
                               FALSE,
                               (GAsyncReadyCallback) swwan_check_status_ready,
                               task);
-    g_object_unref (modem);
 }
 
 static void
@@ -161,84 +167,11 @@ load_connection_status (MMBaseBearer        *bearer,
                         gpointer             user_data)
 {
     load_connection_status_by_cid (MM_BROADBAND_BEARER_CINTERION (bearer),
-                                   mm_broadband_bearer_get_3gpp_cid (MM_BROADBAND_BEARER (bearer)),
+                                   mm_base_bearer_get_profile_id (bearer),
                                    callback,
                                    user_data);
 }
 
-/*****************************************************************************/
-/* Auth helpers */
-
-typedef enum {
-    BEARER_CINTERION_AUTH_UNKNOWN   = -1,
-    BEARER_CINTERION_AUTH_NONE      =  0,
-    BEARER_CINTERION_AUTH_PAP       =  1,
-    BEARER_CINTERION_AUTH_CHAP      =  2,
-    BEARER_CINTERION_AUTH_MSCHAPV2  =  3,
-} BearerCinterionAuthType;
-
-static BearerCinterionAuthType
-parse_auth_type (MMBearerAllowedAuth mm_auth)
-{
-    switch (mm_auth) {
-    case MM_BEARER_ALLOWED_AUTH_NONE:
-        return BEARER_CINTERION_AUTH_NONE;
-    case MM_BEARER_ALLOWED_AUTH_PAP:
-        return BEARER_CINTERION_AUTH_PAP;
-    case MM_BEARER_ALLOWED_AUTH_CHAP:
-        return BEARER_CINTERION_AUTH_CHAP;
-    case MM_BEARER_ALLOWED_AUTH_MSCHAPV2:
-        return BEARER_CINTERION_AUTH_MSCHAPV2;
-    case MM_BEARER_ALLOWED_AUTH_UNKNOWN:
-    case MM_BEARER_ALLOWED_AUTH_MSCHAP:
-    case MM_BEARER_ALLOWED_AUTH_EAP:
-    default:
-        return BEARER_CINTERION_AUTH_UNKNOWN;
-    }
-}
-
-/* AT^SGAUTH=<cid>[, <auth_type>[, <passwd>, <user>]] */
-static gchar *
-build_auth_string (MMBroadbandBearerCinterion *self,
-                   MMBearerProperties         *config,
-                   guint                       cid)
-{
-    const gchar             *user;
-    const gchar             *passwd;
-    gboolean                 has_user;
-    gboolean                 has_passwd;
-    MMBearerAllowedAuth      auth;
-    BearerCinterionAuthType  encoded_auth = BEARER_CINTERION_AUTH_UNKNOWN;
-
-    user   = mm_bearer_properties_get_user         (config);
-    passwd = mm_bearer_properties_get_password     (config);
-    auth   = mm_bearer_properties_get_allowed_auth (config);
-
-    has_user     = (user   && user[0]);
-    has_passwd   = (passwd && passwd[0]);
-    encoded_auth = parse_auth_type (auth);
-
-    /* When 'none' requested, we won't require user/password */
-    if (encoded_auth == BEARER_CINTERION_AUTH_NONE) {
-        if (has_user || has_passwd)
-            mm_obj_warn (self, "APN user/password given but 'none' authentication requested");
-        return g_strdup_printf ("^SGAUTH=%u,%d", cid, encoded_auth);
-    }
-
-    /* No explicit auth type requested? */
-    if (encoded_auth == BEARER_CINTERION_AUTH_UNKNOWN) {
-        /* If no user/passwd given, do nothing */
-        if (!has_user && !has_passwd)
-            return NULL;
-
-        /* If user/passwd given, default to PAP */
-        mm_obj_dbg (self, "APN user/password given but no authentication type explicitly requested: defaulting to 'PAP'");
-        encoded_auth = BEARER_CINTERION_AUTH_PAP;
-    }
-
-    return g_strdup_printf ("^SGAUTH=%u,%d,%s,%s", cid, encoded_auth, passwd, user);
-}
-
 /******************************************************************************/
 /* Dial 3GPP */
 
@@ -373,34 +306,18 @@ dial_3gpp_context_step (GTask *task)
     }
 
     switch (ctx->step) {
-    case DIAL_3GPP_CONTEXT_STEP_FIRST: {
-        MMBearerIpFamily ip_family;
-
-        /* Only IPv4 supported by this bearer implementation for now */
-        ip_family = mm_bearer_properties_get_ip_type (mm_base_bearer_peek_config (MM_BASE_BEARER (ctx->self)));
-        if (ip_family == MM_BEARER_IP_FAMILY_NONE || ip_family == MM_BEARER_IP_FAMILY_ANY) {
-            gchar *ip_family_str;
-
-            ip_family = mm_base_bearer_get_default_ip_family (MM_BASE_BEARER (ctx->self));
-            ip_family_str = mm_bearer_ip_family_build_string_from_mask (ip_family);
-            mm_obj_dbg (self, "no specific IP family requested, defaulting to %s", ip_family_str);
-            g_free (ip_family_str);
-        }
-
-        if (ip_family != MM_BEARER_IP_FAMILY_IPV4) {
-            g_task_return_new_error (task, MM_CORE_ERROR, MM_CORE_ERROR_UNSUPPORTED,
-                                     "Only IPv4 is supported by this modem");
-            g_object_unref (task);
-            return;
-        }
-
+    case DIAL_3GPP_CONTEXT_STEP_FIRST:
         ctx->step++;
-    } /* fall through */
+        /* fall through */
 
     case DIAL_3GPP_CONTEXT_STEP_AUTH: {
         gchar *command;
 
-        command = build_auth_string (self, mm_base_bearer_peek_config (MM_BASE_BEARER (ctx->self)), ctx->cid);
+        command = mm_cinterion_build_auth_string (self,
+                                                  mm_broadband_modem_cinterion_get_family (MM_BROADBAND_MODEM_CINTERION (ctx->modem)),
+                                                  mm_base_bearer_peek_config (MM_BASE_BEARER (ctx->self)),
+                                                  ctx->cid);
+
         if (command) {
             mm_obj_dbg (self, "dial step %u/%u: authenticating...", ctx->step, DIAL_3GPP_CONTEXT_STEP_LAST);
             /* Send SGAUTH write, if User & Pass are provided.
@@ -433,7 +350,7 @@ dial_3gpp_context_step (GTask *task)
         mm_base_modem_at_command_full (ctx->modem,
                                        ctx->primary,
                                        command,
-                                       90,
+                                       MM_BASE_BEARER_DEFAULT_CONNECTION_TIMEOUT,
                                        FALSE,
                                        FALSE,
                                        NULL,
@@ -447,7 +364,7 @@ dial_3gpp_context_step (GTask *task)
         mm_obj_dbg (self, "dial step %u/%u: checking SWWAN interface %u status...",
                     ctx->step, DIAL_3GPP_CONTEXT_STEP_LAST, usb_interface_configs[ctx->usb_interface_config_index].swwan_index);
         load_connection_status_by_cid (ctx->self,
-                                       ctx->cid,
+                                       (gint) ctx->cid,
                                        (GAsyncReadyCallback) dial_connection_status_ready,
                                        task);
         return;
@@ -484,7 +401,7 @@ dial_3gpp (MMBroadbandBearer   *self,
     g_task_set_task_data (task, ctx, (GDestroyNotify) dial_3gpp_context_free);
 
     /* Setup context */
-    ctx->self    = g_object_ref (self);
+    ctx->self    = MM_BROADBAND_BEARER_CINTERION (g_object_ref (self));
     ctx->modem   = g_object_ref (modem);
     ctx->primary = g_object_ref (primary);
     ctx->cid     = cid;
@@ -630,7 +547,7 @@ disconnect_3gpp_context_step (GTask *task)
         mm_base_modem_at_command_full (ctx->modem,
                                        ctx->primary,
                                        command,
-                                       10,
+                                       MM_BASE_BEARER_DEFAULT_DISCONNECTION_TIMEOUT,
                                        FALSE,
                                        FALSE,
                                        NULL,
@@ -645,7 +562,7 @@ disconnect_3gpp_context_step (GTask *task)
                     ctx->step, DISCONNECT_3GPP_CONTEXT_STEP_LAST,
                     usb_interface_configs[ctx->usb_interface_config_index].swwan_index);
         load_connection_status_by_cid (MM_BROADBAND_BEARER_CINTERION (ctx->self),
-                                       ctx->cid,
+                                       (gint) ctx->cid,
                                        (GAsyncReadyCallback) disconnect_connection_status_ready,
                                        task);
          return;
@@ -685,8 +602,8 @@ disconnect_3gpp (MMBroadbandBearer  *self,
     g_task_set_task_data (task, ctx, (GDestroyNotify) disconnect_3gpp_context_free);
 
     /* Setup context */
-    ctx->self    = g_object_ref (self);
-    ctx->modem   = g_object_ref (modem);
+    ctx->self    = MM_BROADBAND_BEARER_CINTERION (g_object_ref (self));
+    ctx->modem   = MM_BASE_MODEM (g_object_ref (modem));
     ctx->primary = g_object_ref (primary);
     ctx->data    = g_object_ref (data);
     ctx->cid     = cid;
@@ -758,6 +675,10 @@ mm_broadband_bearer_cinterion_class_init (MMBroadbandBearerCinterionClass *klass
 
     base_bearer_class->load_connection_status        = load_connection_status;
     base_bearer_class->load_connection_status_finish = load_connection_status_finish;
+#if defined WITH_SYSTEMD_SUSPEND_RESUME
+    base_bearer_class->reload_connection_status        = load_connection_status;
+    base_bearer_class->reload_connection_status_finish = load_connection_status_finish;
+#endif
 
     broadband_bearer_class->dial_3gpp              = dial_3gpp;
     broadband_bearer_class->dial_3gpp_finish       = dial_3gpp_finish;
