@@ -41,7 +41,6 @@
 static void iface_modem_location_init (MMIfaceModemLocation *iface);
 
 #if defined WITH_QMI
-static void iface_modem_init          (MMIfaceModem *iface);
 static void iface_modem_firmware_init (MMIfaceModemFirmware *iface);
 #endif
 
@@ -49,7 +48,6 @@ static MMIfaceModemLocation *iface_modem_location_parent;
 
 G_DEFINE_TYPE_EXTENDED (MMBroadbandModemMbimFoxconn, mm_broadband_modem_mbim_foxconn, MM_TYPE_BROADBAND_MODEM_MBIM, 0,
 #if defined WITH_QMI
-                        G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM, iface_modem_init)
                         G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_FIRMWARE, iface_modem_firmware_init)
 #endif
                         G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_LOCATION, iface_modem_location_init))
@@ -67,59 +65,6 @@ struct _MMBroadbandModemMbimFoxconnPrivate {
 
 #if defined WITH_QMI
 
-/*****************************************************************************/
-/* FCC unlock (Modem interface) */
-
-static gboolean
-fcc_unlock_finish (MMIfaceModem  *self,
-                   GAsyncResult  *res,
-                   GError       **error)
-{
-    return g_task_propagate_boolean (G_TASK (res), error);
-}
-
-static void
-dms_foxconn_set_fcc_authentication_ready (QmiClientDms *client,
-                                          GAsyncResult *res,
-                                          GTask        *task)
-{
-    GError *error = NULL;
-    g_autoptr(QmiMessageDmsFoxconnSetFccAuthenticationOutput) output = NULL;
-
-    output = qmi_client_dms_foxconn_set_fcc_authentication_finish (client, res, &error);
-    if (!output || !qmi_message_dms_foxconn_set_fcc_authentication_output_get_result (output, &error))
-        g_task_return_error (task, error);
-    else
-        g_task_return_boolean (task, TRUE);
-    g_object_unref (task);
-}
-
-static void
-fcc_unlock (MMIfaceModem        *self,
-            GAsyncReadyCallback  callback,
-            gpointer             user_data)
-{
-    GTask     *task;
-    QmiClient *client = NULL;
-    g_autoptr(QmiMessageDmsFoxconnSetFccAuthenticationInput) input = NULL;
-
-    if (!mm_shared_qmi_ensure_client (MM_SHARED_QMI (self),
-                                      QMI_SERVICE_DMS, &client,
-                                      callback, user_data))
-        return;
-
-    task = g_task_new (self, NULL, callback, user_data);
-
-    input = qmi_message_dms_foxconn_set_fcc_authentication_input_new ();
-    qmi_message_dms_foxconn_set_fcc_authentication_input_set_value (input, 0x00, NULL);
-    qmi_client_dms_foxconn_set_fcc_authentication (QMI_CLIENT_DMS (client),
-                                                   input,
-                                                   5,
-                                                   NULL,
-                                                   (GAsyncReadyCallback)dms_foxconn_set_fcc_authentication_ready,
-                                                   task);
-}
-
 /*****************************************************************************/
 /* Firmware update settings
  *
@@ -145,15 +90,29 @@ foxconn_get_firmware_version_ready (QmiClientDms *client,
     GError                                       *error = NULL;
     MMFirmwareUpdateSettings                     *update_settings = NULL;
     const gchar                                  *str;
+    MMIfaceModemFirmware                         *self;
+    guint                                         vendor_id;
+    guint                                         product_id;
 
     output = qmi_client_dms_foxconn_get_firmware_version_finish (client, res, &error);
     if (!output || !qmi_message_dms_foxconn_get_firmware_version_output_get_result (output, &error))
         goto out;
 
-    /* Create update settings now */
-    update_settings = mm_firmware_update_settings_new (MM_MODEM_FIRMWARE_UPDATE_METHOD_FASTBOOT |
-                                                       MM_MODEM_FIRMWARE_UPDATE_METHOD_QMI_PDC);
-    mm_firmware_update_settings_set_fastboot_at (update_settings, "AT^FASTBOOT");
+    /* Create update settings now:
+     * 0x105b is the T99W175 module, T99W175 supports QDU,
+     * T99W265(0x0489:0xe0da ; 0x0489:0xe0db): supports QDU
+     * else support FASTBOOT and QMI PDC.
+     */
+    self = g_task_get_source_object (task);
+    vendor_id = mm_base_modem_get_vendor_id (MM_BASE_MODEM (self));
+    product_id = mm_base_modem_get_product_id (MM_BASE_MODEM (self));
+    if (vendor_id == 0x105b || (vendor_id == 0x0489 && (product_id  == 0xe0da || product_id == 0xe0db)))
+        update_settings = mm_firmware_update_settings_new (MM_MODEM_FIRMWARE_UPDATE_METHOD_MBIM_QDU);
+    else {
+        update_settings = mm_firmware_update_settings_new (MM_MODEM_FIRMWARE_UPDATE_METHOD_FASTBOOT |
+                                                           MM_MODEM_FIRMWARE_UPDATE_METHOD_QMI_PDC);
+        mm_firmware_update_settings_set_fastboot_at (update_settings, "AT^FASTBOOT");
+    }
 
     qmi_message_dms_foxconn_get_firmware_version_output_get_version (output, &str, NULL);
     mm_firmware_update_settings_set_version (update_settings, str);
@@ -178,6 +137,8 @@ firmware_load_update_settings (MMIfaceModemFirmware *self,
     GTask                                       *task;
     QmiMessageDmsFoxconnGetFirmwareVersionInput *input = NULL;
     QmiClient                                   *client = NULL;
+    guint                                        vendor_id;
+    guint                                        product_id;
 
     task = g_task_new (self, NULL, callback, user_data);
 
@@ -192,9 +153,11 @@ firmware_load_update_settings (MMIfaceModemFirmware *self,
         return;
     }
 
+    vendor_id = mm_base_modem_get_vendor_id (MM_BASE_MODEM (self));
+    product_id = mm_base_modem_get_product_id (MM_BASE_MODEM (self));
     input = qmi_message_dms_foxconn_get_firmware_version_input_new ();
-    /* 0x105b is the T99W175 module, T99W175 needs to compare the apps version. */
-    if (mm_base_modem_get_vendor_id (MM_BASE_MODEM (self)) == 0x105b)
+    /* 0x105b is the T99W175 module, T99W175/T99W265 need to compare the apps version. */
+    if (vendor_id == 0x105b || (vendor_id == 0x0489 && (product_id  == 0xe0da || product_id == 0xe0db)))
         qmi_message_dms_foxconn_get_firmware_version_input_set_version_type (
             input,
             QMI_DMS_FOXCONN_FIRMWARE_VERSION_TYPE_FIRMWARE_MCFG_APPS,
@@ -489,17 +452,30 @@ mm_broadband_modem_mbim_foxconn_new (const gchar  *device,
                                      guint16       vendor_id,
                                      guint16       product_id)
 {
+    const gchar *carrier_config_mapping = NULL;
+
+    /* T77W968 (DW5821e is also T77W968) modules use t77w968 carrier mapping table,
+     * T99W175 modules use t99w175 carrier mapping table. */
+    if ((vendor_id == 0x0489 && (product_id == 0xe0b4 || product_id == 0xe0b5)) ||
+        (vendor_id == 0x413c && (product_id == 0x81d7 || product_id == 0x81e0)))
+        carrier_config_mapping = PKGDATADIR "/mm-foxconn-t77w968-carrier-mapping.conf";
+    else if (vendor_id == 0x105b && (product_id == 0xe0ab || product_id == 0xe0b0 || product_id == 0xe0b1))
+        carrier_config_mapping = PKGDATADIR "/mm-foxconn-t99w175-carrier-mapping.conf";
+
     return g_object_new (MM_TYPE_BROADBAND_MODEM_MBIM_FOXCONN,
                          MM_BASE_MODEM_DEVICE,     device,
                          MM_BASE_MODEM_DRIVERS,    drivers,
                          MM_BASE_MODEM_PLUGIN,     plugin,
                          MM_BASE_MODEM_VENDOR_ID,  vendor_id,
                          MM_BASE_MODEM_PRODUCT_ID, product_id,
+                         /* MBIM bearer supports NET only */
+                         MM_BASE_MODEM_DATA_NET_SUPPORTED, TRUE,
+                         MM_BASE_MODEM_DATA_TTY_SUPPORTED, FALSE,
                          MM_IFACE_MODEM_SIM_HOT_SWAP_SUPPORTED,              TRUE,
                          MM_IFACE_MODEM_SIM_HOT_SWAP_CONFIGURED,             FALSE,
                          MM_IFACE_MODEM_PERIODIC_SIGNAL_CHECK_DISABLED,      TRUE,
                          MM_IFACE_MODEM_LOCATION_ALLOW_GPS_UNMANAGED_ALWAYS, TRUE,
-                         MM_IFACE_MODEM_CARRIER_CONFIG_MAPPING,              PKGDATADIR "/mm-foxconn-carrier-mapping.conf",
+                         MM_IFACE_MODEM_CARRIER_CONFIG_MAPPING,              carrier_config_mapping,
                          NULL);
 }
 
@@ -526,13 +502,6 @@ iface_modem_location_init (MMIfaceModemLocation *iface)
 
 #if defined WITH_QMI
 
-static void
-iface_modem_init (MMIfaceModem *iface)
-{
-    iface->fcc_unlock = fcc_unlock;
-    iface->fcc_unlock_finish = fcc_unlock_finish;
-}
-
 static void
 iface_modem_firmware_init (MMIfaceModemFirmware *iface)
 {
