@@ -50,12 +50,14 @@ typedef struct {
     MMModemLocationSource  enabled_sources;
     FeatureSupport         qgps_supported;
     GRegex                *qgpsurc_regex;
+    GRegex                *qlwurc_regex;
 } Private;
 
 static void
 private_free (Private *priv)
 {
     g_regex_unref (priv->qgpsurc_regex);
+    g_regex_unref (priv->qlwurc_regex);
     g_slice_free (Private, priv);
 }
 
@@ -75,6 +77,7 @@ get_private (MMSharedQuectel *self)
         priv->enabled_sources   = MM_MODEM_LOCATION_SOURCE_NONE;
         priv->qgps_supported    = FEATURE_SUPPORT_UNKNOWN;
         priv->qgpsurc_regex     = g_regex_new ("\\r\\n\\+QGPSURC:.*", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
+        priv->qlwurc_regex      = g_regex_new ("\\r\\n\\+QLWURC:.*", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
 
         g_assert (MM_SHARED_QUECTEL_GET_INTERFACE (self)->peek_parent_broadband_modem_class);
         priv->broadband_modem_class_parent = MM_SHARED_QUECTEL_GET_INTERFACE (self)->peek_parent_broadband_modem_class (self);
@@ -120,6 +123,12 @@ mm_shared_quectel_setup_ports (MMBroadbandModem *self)
             ports[i],
             priv->qgpsurc_regex,
             NULL, NULL, NULL);
+
+        /* Ignore +QLWURC */
+        mm_port_serial_at_add_unsolicited_msg_handler (
+            ports[i],
+            priv->qlwurc_regex,
+            NULL, NULL, NULL);
     }
 }
 
@@ -422,10 +431,13 @@ mm_shared_quectel_location_load_capabilities (MMIfaceModemLocation *_self,
  *  1) "+QGPSCFG=\"nmeasrc\",1" will be necessary for getting location data
  *     without the nmea port.
  *  2) may be necessary to set "+QGPSCFG=\"gpsnmeatype\".
+ *  3) QGPSXTRA=1 is necessary to support XTRA assistance data for
+ *     faster GNSS location locks.
  */
 static const MMBaseModemAtCommand gps_startup[] = {
     { "+QGPSCFG=\"outport\",\"usbnmea\"", 3, FALSE, mm_base_modem_response_processor_no_result_continue },
     { "+QGPS=1",                          3, FALSE, mm_base_modem_response_processor_no_result_continue },
+    { "+QGPSXTRA=1",                      3, FALSE, mm_base_modem_response_processor_no_result_continue },
     { NULL }
 };
 
@@ -582,7 +594,7 @@ disable_location_gathering_parent_ready (MMIfaceModemLocation *self,
                                          GAsyncResult         *res,
                                          GTask                *task)
 {
-    GError  *error;
+    GError  *error = NULL;
     Private *priv;
 
     priv = get_private (MM_SHARED_QUECTEL (self));
