@@ -293,7 +293,7 @@ disable_cgps_ready (MMBaseModem  *self,
     if (error)
         g_task_return_error (task, error);
     else {
-        priv->enabled_sources &= source;
+        priv->enabled_sources &= ~source;
         g_task_return_boolean (task, TRUE);
     }
     g_object_unref (task);
@@ -304,7 +304,7 @@ parent_disable_location_gathering_ready (MMIfaceModemLocation *self,
                                          GAsyncResult         *res,
                                          GTask                *task)
 {
-    GError  *error;
+    GError  *error = NULL;
     Private *priv;
 
     priv = get_private (MM_SHARED_SIMTECH (self));
@@ -436,7 +436,7 @@ parent_enable_location_gathering_ready (MMIfaceModemLocation *self,
                                         GAsyncResult         *res,
                                         GTask                *task)
 {
-    GError  *error;
+    GError  *error = NULL;
     Private *priv;
 
     priv = get_private (MM_SHARED_SIMTECH (self));
