@@ -72,6 +72,13 @@ resuming_cb (MMSleepMonitor *sleep_monitor)
     mm_base_manager_start (manager, FALSE);
 }
 
+static void
+resuming_quick_cb (MMSleepMonitor *sleep_monitor)
+{
+    mm_dbg ("syncing modem state (quick resuming)");
+    mm_base_manager_sync (manager);
+}
+
 #endif
 
 static void
@@ -133,23 +140,14 @@ name_lost_cb (GDBusConnection *connection,
 static void
 register_dbus_errors (void)
 {
-  static volatile guint32 aux = 0;
+    /* This method will always return success once during runtime */
+    if (!mm_common_register_errors ())
+        return;
 
-  if (aux)
-      return;
-
-  /* Register all known own errors */
-  aux |= MM_CORE_ERROR;
-  aux |= MM_MOBILE_EQUIPMENT_ERROR;
-  aux |= MM_CONNECTION_ERROR;
-  aux |= MM_SERIAL_ERROR;
-  aux |= MM_MESSAGE_ERROR;
-  aux |= MM_CDMA_ACTIVATION_ERROR;
-
-  /* We no longer use MM_CORE_ERROR_CANCELLED in the daemon, we rely on
-   * G_IO_ERROR_CANCELLED internally */
-  g_dbus_error_unregister_error (MM_CORE_ERROR, MM_CORE_ERROR_CANCELLED, MM_CORE_ERROR_DBUS_PREFIX ".Cancelled");
-  g_dbus_error_register_error   (G_IO_ERROR,    G_IO_ERROR_CANCELLED,    MM_CORE_ERROR_DBUS_PREFIX ".Cancelled");
+    /* We no longer use MM_CORE_ERROR_CANCELLED in the daemon, we rely on
+     * G_IO_ERROR_CANCELLED internally */
+    g_dbus_error_unregister_error (MM_CORE_ERROR, MM_CORE_ERROR_CANCELLED, MM_CORE_ERROR_DBUS_PREFIX ".Cancelled");
+    g_dbus_error_register_error   (G_IO_ERROR,    G_IO_ERROR_CANCELLED,    MM_CORE_ERROR_DBUS_PREFIX ".Cancelled");
 }
 
 int
@@ -168,7 +166,7 @@ main (int argc, char *argv[])
                        mm_context_get_log_timestamps (),
                        mm_context_get_log_relative_timestamps (),
                        &error)) {
-        g_warning ("failed to set up logging: %s", error->message);
+        g_printerr ("error: failed to set up logging: %s\n", error->message);
         g_error_free (error);
         exit (1);
     }
@@ -182,6 +180,9 @@ main (int argc, char *argv[])
     mm_info ("ModemManager (version " MM_DIST_VERSION ") starting in %s bus...",
              mm_context_get_test_session () ? "session" : "system");
 
+    /* Detect runtime charset conversion support */
+    mm_modem_charsets_init ();
+
     /* Acquire name, don't allow replacement */
     name_id = g_bus_own_name (mm_context_get_test_session () ? G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM,
                               MM_DBUS_SERVICE,
@@ -197,7 +198,12 @@ main (int argc, char *argv[])
 
         if (mm_context_get_test_no_suspend_resume())
             mm_dbg ("Suspend/resume support disabled at runtime");
-        else {
+        else if (mm_context_get_test_quick_suspend_resume()) {
+            mm_dbg ("Quick suspend/resume hooks enabled");
+            sleep_monitor = mm_sleep_monitor_get ();
+            g_signal_connect (sleep_monitor, MM_SLEEP_MONITOR_RESUMING, G_CALLBACK (resuming_quick_cb), NULL);
+        } else {
+            mm_dbg ("Full suspend/resume hooks enabled");
             sleep_monitor = mm_sleep_monitor_get ();
             g_signal_connect (sleep_monitor, MM_SLEEP_MONITOR_SLEEPING, G_CALLBACK (sleeping_cb), NULL);
             g_signal_connect (sleep_monitor, MM_SLEEP_MONITOR_RESUMING, G_CALLBACK (resuming_cb), NULL);
