@@ -178,6 +178,15 @@ mm_filter_port (MMFilter        *self,
         }
     }
 
+    /* If this is a QRTR device, we always allow it. This check comes before
+     * checking for VIRTUAL since qrtr devices don't have a sysfs path, and the
+     * check for VIRTUAL will return FALSE. */
+    if ((self->priv->enabled_rules & MM_FILTER_RULE_QRTR) &&
+        g_str_equal (subsystem, "qrtr")) {
+        mm_obj_dbg (self, "(%s/%s) port allowed: qrtr device", subsystem, name);
+        return TRUE;
+    }
+
     /* If this is a virtual device, don't allow it */
     if ((self->priv->enabled_rules & MM_FILTER_RULE_VIRTUAL) &&
         (!mm_kernel_device_get_physdev_sysfs_path (port))) {
@@ -206,29 +215,19 @@ mm_filter_port (MMFilter        *self,
         return TRUE;
     }
 
+    /* If this is a wwan port/device, we always allow it */
+    if ((self->priv->enabled_rules & MM_FILTER_RULE_WWAN) &&
+        (g_strcmp0 (subsystem, "wwan") == 0)) {
+        mm_obj_dbg (self, "(%s/%s) port allowed: wwan device", subsystem, name);
+        return TRUE;
+    }
+
     /* If this is a tty device, we may allow it */
     if ((self->priv->enabled_rules & MM_FILTER_RULE_TTY) &&
         (g_strcmp0 (subsystem, "tty") == 0)) {
         const gchar *physdev_subsystem;
         const gchar *driver;
 
-        /* Blacklist rules first */
-
-        /* Ignore blacklisted tty devices. */
-        if ((self->priv->enabled_rules & MM_FILTER_RULE_TTY_BLACKLIST) &&
-            (mm_kernel_device_get_global_property_as_boolean (port, ID_MM_TTY_BLACKLIST))) {
-            mm_obj_dbg (self, "(%s/%s): port filtered: tty is blacklisted", subsystem, name);
-            return FALSE;
-        }
-
-        /* Is the device in the manual-only greylist? If so, return if this is an
-         * automatic scan. */
-        if ((self->priv->enabled_rules & MM_FILTER_RULE_TTY_MANUAL_SCAN_ONLY) &&
-            (!manual_scan && mm_kernel_device_get_global_property_as_boolean (port, ID_MM_TTY_MANUAL_SCAN_ONLY))) {
-            mm_obj_dbg (self, "(%s/%s): port filtered: tty probed only in manual scan", subsystem, name);
-            return FALSE;
-        }
-
         /* Mixed blacklist/whitelist rules */
 
         /* If the physdev is a 'platform' or 'pnp' device that's not whitelisted, ignore it */
@@ -242,12 +241,6 @@ mm_filter_port (MMFilter        *self,
             return FALSE;
         }
 
-        /* Default allowed? */
-        if (self->priv->enabled_rules & MM_FILTER_RULE_TTY_DEFAULT_ALLOWED) {
-            mm_obj_dbg (self, "(%s/%s) port allowed", subsystem, name);
-            return TRUE;
-        }
-
         /* Whitelist rules last */
 
         /* If the TTY kernel driver is one expected modem kernel driver, allow it */
@@ -421,11 +414,8 @@ log_object_build_id (MMLogObject *_self)
 
 /*****************************************************************************/
 
-/* If TTY rule enabled, either DEFAULT_ALLOWED or DEFAULT_FORBIDDEN must be set. */
-#define VALIDATE_RULE_TTY(rules) (!(rules & MM_FILTER_RULE_TTY) || \
-                                  ((rules & (MM_FILTER_RULE_TTY_DEFAULT_ALLOWED | MM_FILTER_RULE_TTY_DEFAULT_FORBIDDEN)) && \
-                                   ((rules & (MM_FILTER_RULE_TTY_DEFAULT_ALLOWED | MM_FILTER_RULE_TTY_DEFAULT_FORBIDDEN)) != \
-                                    (MM_FILTER_RULE_TTY_DEFAULT_ALLOWED | MM_FILTER_RULE_TTY_DEFAULT_FORBIDDEN))))
+/* If TTY rule enabled, DEFAULT_FORBIDDEN must be set. */
+#define VALIDATE_RULE_TTY(rules) (!(rules & MM_FILTER_RULE_TTY) || (rules & (MM_FILTER_RULE_TTY_DEFAULT_FORBIDDEN)))
 
 MMFilter *
 mm_filter_new (MMFilterRule   enabled_rules,
@@ -453,21 +443,19 @@ mm_filter_new (MMFilterRule   enabled_rules,
     mm_obj_dbg (self, "  explicit whitelist:         %s", RULE_ENABLED_STR (MM_FILTER_RULE_EXPLICIT_WHITELIST));
     mm_obj_dbg (self, "  explicit blacklist:         %s", RULE_ENABLED_STR (MM_FILTER_RULE_EXPLICIT_BLACKLIST));
     mm_obj_dbg (self, "  plugin whitelist:           %s", RULE_ENABLED_STR (MM_FILTER_RULE_PLUGIN_WHITELIST));
+    mm_obj_dbg (self, "  qrtr devices allowed:       %s", RULE_ENABLED_STR (MM_FILTER_RULE_QRTR));
     mm_obj_dbg (self, "  virtual devices forbidden:  %s", RULE_ENABLED_STR (MM_FILTER_RULE_VIRTUAL));
     mm_obj_dbg (self, "  net devices allowed:        %s", RULE_ENABLED_STR (MM_FILTER_RULE_NET));
     mm_obj_dbg (self, "  usbmisc devices allowed:    %s", RULE_ENABLED_STR (MM_FILTER_RULE_USBMISC));
     mm_obj_dbg (self, "  rpmsg devices allowed:      %s", RULE_ENABLED_STR (MM_FILTER_RULE_RPMSG));
+    mm_obj_dbg (self, "  wwan devices allowed:       %s", RULE_ENABLED_STR (MM_FILTER_RULE_WWAN));
     if (self->priv->enabled_rules & MM_FILTER_RULE_TTY) {
         mm_obj_dbg (self, "  tty devices:");
-        mm_obj_dbg (self, "      blacklist applied:        %s", RULE_ENABLED_STR (MM_FILTER_RULE_TTY_BLACKLIST));
-        mm_obj_dbg (self, "      manual scan only applied: %s", RULE_ENABLED_STR (MM_FILTER_RULE_TTY_MANUAL_SCAN_ONLY));
         mm_obj_dbg (self, "      platform driver check:    %s", RULE_ENABLED_STR (MM_FILTER_RULE_TTY_PLATFORM_DRIVER));
         mm_obj_dbg (self, "      driver check:             %s", RULE_ENABLED_STR (MM_FILTER_RULE_TTY_DRIVER));
         mm_obj_dbg (self, "      cdc-acm interface check:  %s", RULE_ENABLED_STR (MM_FILTER_RULE_TTY_ACM_INTERFACE));
         mm_obj_dbg (self, "      with net check:           %s", RULE_ENABLED_STR (MM_FILTER_RULE_TTY_WITH_NET));
-        if (self->priv->enabled_rules & MM_FILTER_RULE_TTY_DEFAULT_ALLOWED)
-            mm_obj_dbg (self, "      default:                  allowed");
-        else if (self->priv->enabled_rules & MM_FILTER_RULE_TTY_DEFAULT_FORBIDDEN)
+        if (self->priv->enabled_rules & MM_FILTER_RULE_TTY_DEFAULT_FORBIDDEN)
             mm_obj_dbg (self, "      default:                  forbidden");
         else
             g_assert_not_reached ();
