@@ -21,6 +21,8 @@
 #include <ModemManager.h>
 #include <libqmi-glib.h>
 
+#include "mm-port.h"
+
 /*****************************************************************************/
 /* QMI/DMS to MM translations */
 
@@ -112,12 +114,25 @@ MMSmsState mm_sms_state_from_qmi_message_tag (QmiWmsMessageTagType tag);
 /*****************************************************************************/
 /* QMI/WDS to MM translations */
 
-QmiWdsAuthentication mm_bearer_allowed_auth_to_qmi_authentication   (MMBearerAllowedAuth auth);
+QmiWdsAuthentication mm_bearer_allowed_auth_to_qmi_authentication   (MMBearerAllowedAuth   auth,
+                                                                     gpointer              log_object,
+                                                                     GError              **error);
 MMBearerAllowedAuth  mm_bearer_allowed_auth_from_qmi_authentication (QmiWdsAuthentication auth);
 MMBearerIpFamily     mm_bearer_ip_family_from_qmi_ip_support_type   (QmiWdsIpSupportType ip_support_type);
 MMBearerIpFamily     mm_bearer_ip_family_from_qmi_pdp_type          (QmiWdsPdpType pdp_type);
 gboolean             mm_bearer_ip_family_to_qmi_pdp_type            (MMBearerIpFamily  ip_family,
                                                                      QmiWdsPdpType    *out_pdp_type);
+QmiWdsApnTypeMask    mm_bearer_apn_type_to_qmi_apn_type             (MMBearerApnType apn_type,
+                                                                     gpointer        log_object);
+MMBearerApnType      mm_bearer_apn_type_from_qmi_apn_type           (QmiWdsApnTypeMask apn_type);
+
+GError *qmi_mobile_equipment_error_from_verbose_call_end_reason_3gpp (QmiWdsVerboseCallEndReason3gpp vcer_3gpp,
+                                                                      gpointer                       log_object);
+
+/*****************************************************************************/
+/* QMI/WDA to MM translations */
+
+QmiDataEndpointType mm_port_subsys_to_qmi_endpoint_type (MMPortSubsys subsys);
 
 /*****************************************************************************/
 /* QMI/OMA to MM translations */
@@ -176,4 +191,14 @@ gboolean mm_qmi_uim_get_card_status_output_parse (gpointer
 /* UIM Get Slot Status parsing */
 gchar *mm_qmi_uim_decode_eid (const gchar *eid, gsize eid_len);
 
+/*****************************************************************************/
+/* UIM Get Configuration parsing */
+
+gboolean mm_qmi_uim_get_configuration_output_parse (gpointer                              log_object,
+                                                    QmiMessageUimGetConfigurationOutput  *output,
+                                                    MMModem3gppFacility                  *o_lock,
+                                                    GError                              **error);
+
+QmiUimCardApplicationPersonalizationFeature qmi_personalization_feature_from_mm_modem_3gpp_facility (MMModem3gppFacility facility);
+
 #endif  /* MM_MODEM_HELPERS_QMI_H */
