@@ -796,6 +796,12 @@ mm_plugin_supports_port (MMPlugin            *self,
         if (self->priv->at)
             probe_run_flags |= MM_PORT_PROBE_AT;
     }
+#if defined WITH_QRTR
+    else if (g_str_equal (mm_kernel_device_get_subsystem (port), "qrtr")) {
+        if (self->priv->qmi)
+            probe_run_flags |= MM_PORT_PROBE_QMI;
+    }
+#endif
 
     /* For potential AT ports, check for more things */
     if (probe_run_flags & MM_PORT_PROBE_AT) {
