@@ -34,7 +34,7 @@
 #include "libqcdm/src/utils.h"
 #include "libqcdm/src/com.h"
 #include "libqcdm/src/errors.h"
-#include "mm-log.h"
+#include "mm-log-test.h"
 
 typedef struct {
     int master;
@@ -54,7 +54,7 @@ wait_for_child (TestData *d, guint32 timeout)
         status = 0;
         ret = waitpid (d->child, &status, WNOHANG);
         g_get_current_time (&now);
-        if (d->child && (now.tv_sec - start.tv_sec > timeout)) {
+        if (d->child && (now.tv_sec - start.tv_sec > (glong)timeout)) {
             /* Kill it */
             if (g_test_verbose ())
                 g_message ("Killing running child process %d", d->child);
@@ -72,7 +72,7 @@ wait_for_child (TestData *d, guint32 timeout)
 static void
 print_buf (const char *detail, const char *buf, gsize len)
 {
-    int i = 0;
+    guint i = 0;
     gboolean newline = FALSE;
 
     g_print ("%s (%zu)  ", detail, len);
@@ -116,7 +116,7 @@ server_wait_request (int fd, char *buf, gsize len)
     struct timeval timeout = { 1, 0 };
     char readbuf[1024];
     ssize_t bytes_read;
-    int total = 0, retries = 0;
+    guint total = 0, retries = 0;
     gsize decap_len = 0;
 
     FD_ZERO (&in);
@@ -213,9 +213,6 @@ qcdm_test_child (int fd, GAsyncReadyCallback cb)
     gboolean success;
     GError *error = NULL;
 
-    /* In the child */
-    g_type_init ();
-
     loop = g_main_loop_new (NULL, FALSE);
 
     port = mm_port_serial_qcdm_new_fd (fd);
@@ -280,7 +277,8 @@ qcdm_verinfo_expect_fail_cb (MMPortSerialQcdm *port,
 
     response = mm_port_serial_qcdm_command_finish (port, res, &error);
 
-    g_assert_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED);
+    /* Expect any kind of error */
+    g_assert (error != NULL);
     g_error_free (error);
     g_assert (response == NULL);
 
@@ -439,26 +437,6 @@ test_pty_cleanup (TestData *d)
     }
 }
 
-void
-_mm_log (const char *loc,
-         const char *func,
-         guint32 level,
-         const char *fmt,
-         ...)
-{
-#if defined ENABLE_TEST_MESSAGE_TRACES
-    /* Dummy log function */
-    va_list args;
-    gchar *msg;
-
-    va_start (args, fmt);
-    msg = g_strdup_vprintf (fmt, args);
-    va_end (args);
-    g_print ("%s\n", msg);
-    g_free (msg);
-#endif
-}
-
 typedef void (*TCFunc) (TestData *, gconstpointer);
 #define TESTCASE_PTY(s, t) g_test_add (s, TestData, NULL, (TCFunc)test_pty_create, (TCFunc)t, (TCFunc)test_pty_cleanup);
 
